-- Zlock
-- adapted from ANALog ClOCK
-- by zorg @ 2014

debug = false
debugTime = 0
selClass = 0
showDates = false
cirnoMode = false
cirnoTimer = 0

screen = {
		width = love.graphics.getWidth(),
		height = love.graphics.getHeight()
	}

-- minima of screen dimensions
local imf = math.min(screen.width, screen.height)

-- so when resizing, the circle will stay a circle,
-- and will fit into the space neatly
radius = (imf / 2) - (imf / 32)

function calcRadius(self)
	return radius * self.reach
end

function calcCoords(self)
	local x = self.radius * math.cos(self.angle) + (screen.width / 2)
	local y = self.radius * math.sin(self.angle) + (screen.height / 2)
	return x, y
end

function syncTime()
	--print('syncing!')
	local h,m,s = os.date('%H'),os.date('%M'),os.date('%S')
	return ((h*3600) + (m*60) + s) % 86400
end

---------------------------------------

function love.load()

	love.graphics.setBackgroundColor(0, 16, 32, 255)

	-- todo: implement textured clockface on canvas or something, either doing a subtractive blend or with meshes
	cClockFace = createCircle(love.graphics.newImage('cclockface.png'),nil,nil,nil,64)
	c_ = love.graphics.newImage('c_.png')

	hands = {}
	dots = {}
	dates = {}

	-- new earth time / degree clock hands
	local degs = require "degs"
	for i,v in ipairs(degs) do
		hands[#hands+1] = v
		hands[#hands].class = 1
	end
	dots[#dots+1] = degs.dots

	-- standard clock hands
	local standard = require "standard"
	for i,v in ipairs(standard) do
		hands[#hands+1] = v
		hands[#hands].class = 2
	end
	dots[#dots+1] = standard.dots 

	-- french revolutionary / decimal / swatch clock hands
	local deci = require "deci"
	for i,v in ipairs(deci) do
		hands[#hands+1] = v
		hands[#hands].class = 3
	end
	dots[#dots+1] = deci.dots

	-- dozenal clock hands
	local dozen = require "dozen"
	for i,v in ipairs(dozen) do
		hands[#hands+1] = v
		hands[#hands].class = 4
	end
	dots[#dots+1] = dozen.dots

	-- base 2^n clock hands
	local hexa = require "hexa"
	for i,v in ipairs(hexa) do
		hands[#hands+1] = v
		hands[#hands].class = 5
	end
	dots[#dots+1] = hexa.dots

	-- mayan base20 clock hands
	local maya = require "maya"
	for i,v in ipairs(maya) do
		hands[#hands+1] = v
		hands[#hands].class = 6
	end
	dots[#dots+1] = maya.dots

	-- date things
	ddate = require "ddate"
	etsep = require "etsep"
	local ord = (tostring(os.date("%d")):sub(-1,-1) == '1')
			and
				'st'
			or
				((tostring(os.date("%d")):sub(-1,-1) == '2')
			and
				'nd'
			or
				((tostring(os.date("%d")):sub(-1,-1) == '3')
			and
				'rd'
			or
				'th'
			))
	dates[1] = os.date("Today is %A; %B %d" .. ord .. ", %Y")
	dates[2] = ddate()
	dates[3] = etsep()

	for i,v in ipairs(hands) do
		v.radius = calcRadius(v)
	end

	if debug then
		accum = debugTime
	else
		accum = 0 + syncTime()
	end

end

-- accum holds the time passed since a midnight, its value is bounded: [0,86400)
function love.update(dt)

	if debug then
		accum = accum --+ dt
	else
		accum = accum + dt
		if accum > 86400 then
			local day = tostring(os.date("%d"))
			dates[1] = os.date("Today is %A; %B %d"
				.. ((day:sub(-1,-1) == '1')
				and 'st'
				or ((day:sub(-1,-1) == '2')
					and 'nd'
					or ((day:sub(-1,-1) == '3')
						and 'rd'
						or 'th')))
				.. ", %Y")
			dates[2] = ddate()
			dates[3] = etsep()
			accum = accum % 86400
		end
	end

	-- sync time
	if dt > 1/10 then
		-- get current time, and set whole part of accum to match it (or not, graphics will jump due to the syncing anyway)
		if not debug then accum = syncTime() end
	end

	-- update hands
	for i,v in ipairs(hands) do
		v:update(accum)
	end

	-- :3
	cirnoTimer = math.max(cirnoTimer - (dt) * (9/cirnoTimer),0)
	--print(cirnoTimer)

end

function love.draw()

	-- face
	if cirnoMode == true then
		love.graphics.setBlendMode('alpha')
		love.graphics.setColor(0,48,127,255)
		love.graphics.circle("fill", screen.width / 2, screen.height / 2, radius)
		love.graphics.setColor(255,255,255,255)
		love.graphics.draw(cClockFace, screen.width / 2, screen.height / 2, 0, radius, radius)
	else
		love.graphics.setColor(0,32,64,255)
		love.graphics.circle("fill", screen.width / 2, screen.height / 2, radius)
	end

	-- dots
	if not cirnoMode then
		if selClass ~= 0 then
			for i,v in ipairs(dots[selClass]) do
				love.graphics.setColor(v.color)
				love.graphics.circle("fill", v.x, v.y, v.radius)
			end
		else
			for j,w in ipairs(dots) do
				for i,v in ipairs(w) do
					love.graphics.setColor(v.color)
					love.graphics.circle("fill", v.x, v.y, v.radius)
				end
			end
		end
	end
	if cirnoMode then
		--dots?
		love.graphics.setColor(255,255,255,255)
		love.graphics.setLineWidth(1)
		for i=1,9 do
			local angle = (((math.pi * 2) / 9) * i) - (math.pi / 2)
			local x,y,r
			if i % 9 == 0 then
				x = ((radius / 9) * (9-0.09)) * math.cos(angle) + (screen.width / 2)
				y = ((radius / 9) * (9-0.09)) * math.sin(angle) + (screen.height / 2)
				r = radius * 0.009
			else
				x = ((radius / 9.9) * 9) * math.cos(angle) + (screen.width / 2)
				y = ((radius / 9.9) * 9) * math.sin(angle) + (screen.height / 2)
				r = radius * 0.009
			end
			love.graphics.circle("line", x, y, r)
		end
	end

	-- hands
	love.graphics.setBlendMode('additive')
	for i,v in ipairs(hands) do
		if selClass == 0 or v.class == selClass then
			v:draw()
		end
	end
	love.graphics.setBlendMode('alpha')

	-- digit stuff
	love.graphics.setColor(255,255,255,191)

	--if selClass == 0 then

		-- elapsed seconds and 10th seconds
		local s1 = tostring(math.floor(accum))
		local s2 = tostring(math.floor(accum*10)%10)
		local s = s1 .. '.' .. s2
		love.graphics.print(
			s,
			screen.width / 2 - love.graphics.getFont():getWidth(s) / 2, 
			screen.height / 2 + love.graphics.getFont():getHeight() * 7 * (radius / 1000)
		)

	--else
	if selClass ~= 0 then
		assert(type(accum) == 'number')
		dots[selClass]:drawinfo(accum)

	end

	-- show dates
	if showDates then
		for i,v in ipairs(dates) do
			love.graphics.setColor(255,255,255,255)
			love.graphics.printf(v,0,(i-1)*love.graphics.getFont():getHeight()*2,screen.width, "left")
		end
	end

	--baka
	if cirnoMode then
		-- mischievous math teacherine
		love.graphics.setColor(255,255,255,cirnoTimer <=9 and 255-(cirnoTimer/9*255) or 0)
		love.graphics.draw(c_, 0, 0, 0, imf/512, imf/512)
	end

end

function love.keypressed(key)
	if key == "escape" then love.event.quit() end
	if key == '0' or key == '`' then selClass = 0 end
	if key == '1' then selClass = 1 end
	if key == '2' then selClass = 2 end
	if key == '3' then selClass = 3 end
	if key == '4' then selClass = 4 end
	if key == '5' then selClass = 5 end
	if key == '6' then selClass = 6 end
	if key == '7' then --[[selClass = 7--]] end -- Unimplemented Chinese System
	if key == '8' then showDates = not showDates end -- THE DATES! THE BEES!
	if key == 'b' then
		local w, h, f = love.window.getMode()
		f.vsync = false
		f.minwidth = 512
		f.minheight = 512
		if f.borderless == true then
			f.centered = false
			f.borderless = false
			f.resizable = true
		else
			f.centered = false
			f.borderless = true
			f.resizable = false
		end
		love.window.setMode(w,h,f)
	end
	if key == '9' then cirnoMode = not cirnoMode; cirnoTimer = 9+math.sqrt(9) end -- :3
end

function love.resize(w, h)
	screen.width, screen.height = w, h

	imf = math.min(screen.width, screen.height)
	radius = (imf / 2) - (imf / 32)

	-- fix clock hands
	for i,v in ipairs(hands) do
		v.radius = calcRadius(v)
	end

	-- fix dots
	for i,v in ipairs(dots) do
		for j,w in ipairs(v) do
			w.radius = calcRadius(w)
			w.x = ((radius / w.o) * w.i) * math.cos(w.angle) + (screen.width / 2)
			w.y = ((radius / w.o) * w.i) * math.sin(w.angle) + (screen.height / 2)
		end
	end

end

---------------------------------------

createCircle = function(image, quad, centerColor, edgeColor, segments)
	local iw,ih = image and image:getWidth() or 1, image and image:getHeight() or 1
	local quad = quad
	if not quad then quad = {['left'] = 0, ['top'] = 0, ['width'] = iw, ['height'] = ih} end
	--print(quad)
	local segments = math.min(math.max((segments or 32), 8), 64)

	local vertices = {}
	
	-- restrict UV-s to the quad's domain
	-- [0,1],[0,1] -> [left,left+width],[top,top+width] - divided by the image's dimensions to stay between [0,1] of course
	local wu = 1.0 / iw
	local hv = 1.0 / ih
	local lu = (quad.left * wu)
	local tv = (quad.top * hv)

	local u = function(u) --[[print(lu, u, wu, lu + u * (quad.width * wu))--]] return lu + u * (quad.width * wu) end
	local v = function(v) --[[print(tv, v, hv, tv + v * (quad.height * hv))--]] return tv + v * (quad.height * hv) end

	-- The first vertex is at the center. We're centering the circle around the origin (0, 0).
	if centerColor or edgeColor then
		table.insert(vertices, {0, 0, u(0.5), v(0.5), centerColor[1] or 255, centerColor[2] or 255, centerColor[3] or 255, centerColor[4] or 255})
	else
		-- don't use per-vertex coloring
		table.insert(vertices, {0, 0, u(0.5), v(0.5)})
	end
	
	-- Create the vertices at the edge of the circle.
	for i=0, segments do
		local angle = (i / segments) * math.pi * 2

		-- Unit-circle.
		local x = math.cos(angle)
		local y = math.sin(angle)

		-- Our position is in the range of [-1, 1] but we want the texture coordinate to be in the range of [0, 1].
		local U = (x + 1) * 0.5
		local V = (y + 1) * 0.5
		
		if centerColor or edgeColor then
			table.insert(vertices, {x, y, u(U), v(V), edgeColor[1] or 255, edgeColor[2] or 255, edgeColor[3] or 255, edgeColor[4] or 255})
		else
			-- don't use per-vertex coloring
			table.insert(vertices, {x, y, u(U), v(V)})
		end
	end
	
	-- The "fan" draw mode is perfect for our circle.
	return love.graphics.newMesh(vertices, image or nil, "fan")
end
